package com.asreader.p3xu.view;

import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.asreader.R;
import com.asreader.p3xu.AsReaderP3xU;
import com.asreader.p3xu.diagnostics.AsReaderP3xUASRLog;
import com.asreader.p3xu.rfid.exception.AsReaderP3xUException;
import com.asreader.p3xu.rfid.type.AsReaderP3xUBuzzerState;
import com.asreader.p3xu.rfid.type.AsReaderP3xUCommandType;
import com.asreader.p3xu.rfid.type.AsReaderP3xULbtItem;
import com.asreader.p3xu.rfid.type.AsReaderP3xUResultCode;
import com.asreader.support.andro.app.rfid.adapter.SpinnerAdapter;
import com.asreader.support.andro.app.rfid.adapter.LbtAdapter;
import com.asreader.support.andro.app.rfid.dialog.WaitDialog;
import com.asreader.support.andro.app.rfid.util.ResUtil;
import com.asreader.support.andro.app.rfid.util.UserConfigs;
import com.asreader.support.andro.app.rfid.view.base.RfidPowerGainActivity;

import java.util.Locale;

public class OptionActivity extends RfidPowerGainActivity implements View.OnClickListener {
    public enum DeviceModel {
        P3xU, D3;
    }
    private static final String TAG = OptionActivity.class.getSimpleName();

    private static final int EVENT_LOAD_OPTION = 1000;
    public static final String APP_NAME = "rfid.reader.demo";
    private static final int DEFAULT_BATTERY_INTERVAL = 10;
    private static final int MIN_AUTO_OFF_TIME = 0;


    private TextView txtBuzzer;
    private TextView txtInventoryTime;
    private TextView txtIdleTime;
    private TextView txtQValue;
    private Button btnSave;
    private Button btnDefault;
    private TextView txtFrequencyAutomatic;
    private Button btnLbtChannel;
    private boolean mIsEnabled;

    private AsReaderP3xUBuzzerState mBuzzer;
    private int mInventoryTime;
    private int mIdleTime;
    private int mQValue;
    private int showFrequency;

    private boolean mIsSaveTable;
    private AsReaderP3xULbtItem[] mFreqTable;
    private boolean mIsSaved;
    private boolean mIsDefault;
    private LinearLayout layoutLBT;
    private LinearLayout layoutInventoryTime;
    private LinearLayout layoutIdleTime;
    private LinearLayout layoutFrequency;
    private boolean mFrequencyAutomatic;
    private DeviceModel deviceModel;
    private int qvalueMin;
    private int qvalueMax;

    private Handler mEvent;

    private String[] mLogLevelText = new String[]{"ERROR", "INFO", "DEBUG", "API", "LIB1", "LIB2", "WATCH", "UTIL", "IO", "DUMP"};
    private UserConfigs mUserConfigs;
    private String mGlobalBandType;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_option);
        mUserConfigs = UserConfigs.getInstance(this.getApplicationContext());
        ActionBar actionBar = getActionBar();
        if (actionBar != null) {
            actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
            actionBar.setCustomView(R.layout.activity_title);
            actionBar.setDisplayHomeAsUpEnabled(true);
            actionBar.setDisplayShowHomeEnabled(true);
        }

        TextView titleText = (TextView) findViewById(R.id.title_text);
        titleText.setText(R.string.option_name);
        final int version = Build.VERSION.SDK_INT;
        if (version >= Build.VERSION_CODES.M)
            titleText.setTextColor(getColor(getApplicationContext(), R.color.black));

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onCreate()");

        mIsSaveTable = false;
        mIsSaved = false;
        mIsDefault = false;
        mFrequencyAutomatic = true;

        // Create Reader
        createAsReaderP3xU();

        SharedPreferences prefs = getSharedPreferences(APP_NAME, MODE_PRIVATE);

        // Initialize Widget
        initWidgets();

        // Initialize Reader
        initAsReaderP3xU();

        // Enable action widget
        enableActionWidgets(false);

        mEvent = new Handler();
    }

    @Override
    protected void onDestroy() {

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onDestroy()");

        // Destroy Reader
        destroyAsReaderP3xU();
        super.onDestroy();
    }

    @Override
    protected void onStart() {
        super.onStart();
        Log.v(TAG,"----onStart--------");
        // Load Option
        WaitDialog.show(this, R.string.load_option);
        new Thread(mLoadOption).start();

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onStart()");
    }

    @Override
    protected void onStop() {

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onStart()");
        super.onStop();
    }

    @Override
    protected void onResume() {
        super.onResume();
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onResume()");
    }

    @Override
    protected void onPause() {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onPause()");
        super.onPause();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        switch (item.getItemId()) {
            case android.R.id.home:
                if (mIsSaved) {
                    this.setResult(Activity.RESULT_OK, null);
                } else {
                    this.setResult(Activity.RESULT_CANCELED);
                }
                this.finish();

                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onBackPressed() {

        if (mIsSaved) {
            this.setResult(Activity.RESULT_OK, null);
        } else {
            this.setResult(Activity.RESULT_CANCELED);
        }

        super.onBackPressed();
    }

    @Override
    public void onClick(View v) {

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));
        mIsDefault = (v.getId() == R.id.option_default);

        switch (v.getId()) {
            case R.id.buzzer:
                showBuzzerDialog();
                break;
            case R.id.inventory_time:
                showInventoryTimeDialog();
                break;
            case R.id.idle_time:
                showIdleTimeDialog();
                break;
            case R.id.q_value:
                showQValueDialog();
                break;
            case R.id.frequency_automatic:
                showFrequencyAutomaticDialog();
                break;
            case R.id.btn_lbt_channel:
                WaitDialog.show(this, R.string.load_frequency);
                new Thread(mLoadFrequencyTable).start();
                break;
            case R.id.save:
                mIsSaved = true;
                // Enable action widget
                enableActionWidgets(false);
                // Save Option
                WaitDialog.show(this, R.string.save_option);
                new Thread(mSaveOption).start();
                break;
            case R.id.option_default:
                enableActionWidgets(false);
                WaitDialog.show(this, R.string.load_default_option);
                mDefaultOption();
                break;
        }
    }

    @Override
    protected void clearWidgets() {
    }

    // Asynchronous Load Option
    private Runnable mLoadOption = new Runnable() {

        @Override
        public void run() {
            AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "+++ INFO. $mLoadOption.run()");
            try {
                Thread.sleep(100);
                String devieModel = getAsReaderP3xU().getDeviceModel();
                if (devieModel.equals("0")) {
                    deviceModel = DeviceModel.P3xU;
                } else if (devieModel.equals("1")) {
                    //D3
                    deviceModel = DeviceModel.D3;
                }
            } catch (Exception e) {
                deviceModel = DeviceModel.D3;
                AsReaderP3xUASRLog.e(TAG, "ERROR. showQValueDialog().getDeviceModel", e.getMessage());
            }

            try {
                Thread.sleep(100);
                qvalueMin = getAsReaderP3xU().getQValueMin();
            } catch (Exception e) {
                qvalueMin = 0;
                AsReaderP3xUASRLog.e(TAG, "ERROR. showQValueDialog().getQValueMin", e.getMessage());
            }

            try {
                Thread.sleep(100);
                qvalueMax = getAsReaderP3xU().getQValueMax();
            } catch (Exception e) {
                qvalueMax = 8;
                AsReaderP3xUASRLog.e(TAG, "ERROR. showQValueDialog().getQValueMax", e.getMessage());
            }

            try {
                Thread.sleep(100);
                mBuzzer = getAsReaderP3xU().getBuzzer();
                mUserConfigs.setVolume(mBuzzer);
                mUserConfigs.Save();
            } catch (AsReaderP3xUException | InterruptedException e) {
                AsReaderP3xUASRLog.e(TAG, e, "ERROR. $mLoadOption.run() - Failed to get buzzer");
                mBuzzer = AsReaderP3xUBuzzerState.High;
            }

            if (deviceModel == DeviceModel.P3xU) {
                try {
                    Thread.sleep(100);
                    mInventoryTime = getAsReaderP3xU().getInventoryTime();
                    mUserConfigs.setInventoryTime(mInventoryTime);
                    mUserConfigs.Save();
                } catch (AsReaderP3xUException | InterruptedException e) {
                    AsReaderP3xUASRLog.e(TAG, e, "ERROR. $mLoadOption.run() - Failed to get inventory time");
                    mInventoryTime = 100;
                }

                try {
                    Thread.sleep(100);
                    mIdleTime = getAsReaderP3xU().getIdleTime();
                    mUserConfigs.setIdleTime(mIdleTime);
                    mUserConfigs.Save();
                } catch (AsReaderP3xUException | InterruptedException e) {
                    AsReaderP3xUASRLog.e(TAG, e, "ERROR. $mLoadOption.run() - Failed to get idle time");
                    mIdleTime = 400;
                }
            }

            try {
                Thread.sleep(100);
                mQValue = getAsReaderP3xU().getQValue();
                mUserConfigs.setQValue(mQValue);
                mUserConfigs.Save();
            } catch (AsReaderP3xUException | InterruptedException e2) {
                AsReaderP3xUASRLog.e(TAG, e2, "ERROR. $mLoadOption.run() - Failed to get Q value");
                mQValue = 4;
            }

            try {
                mGlobalBandType = getAsReaderP3xU().getRegion();

            } catch (AsReaderP3xUException e2) {
                AsReaderP3xUASRLog.e(TAG, e2, "ERROR. $mLoadOption.run() - Failed to get global band type");
                mGlobalBandType = "Unknown";
            }

            // Get Frequency Automatic
            try {
                mFrequencyAutomatic = getAsReaderP3xU().getFrequencyAutomatic();
            } catch (AsReaderP3xUException e) {
                AsReaderP3xUASRLog.e(TAG, e, "ERROR. initReader() - Failed to get frequency automatic");
                mFrequencyAutomatic = true;
            }

            mUserConfigs.setFrequencyAuto(mFrequencyAutomatic);
            mUserConfigs.Save();
            mIsSaveTable = false;

            mEvent.post(new Runnable() {

                @Override
                public void run() {
                    displayBuzzer();
                    if (deviceModel == DeviceModel.P3xU) {
                        layoutInventoryTime.setVisibility(View.VISIBLE);
                        layoutIdleTime.setVisibility(View.VISIBLE);
                        displayInventoryTime();
                        displayIdleTime();
                    } else {
                        layoutInventoryTime.setVisibility(View.GONE);
                        layoutIdleTime.setVisibility(View.GONE);
                    }
                    displayQValue();
                    displayFrequencyTable();

                    if (isJapaneseType())
                        layoutFrequency.setVisibility(View.VISIBLE);
                    else
                        layoutFrequency.setVisibility(View.GONE);

                    WaitDialog.hide();
                    if (mIsDefault) {
                        AlertDialog.Builder builder = new AlertDialog.Builder(OptionActivity.this);
                        builder.setTitle("Success");
                        builder.setMessage("Set to default successfully")
                                .setCancelable(false)
                                .setPositiveButton("OK", null);

                        AlertDialog dlg = builder.create();
                        dlg.show();
                        mIsDefault = false;
                    }
                    enableActionWidgets(true);
                }

            });
            mEvent.sendEmptyMessage(EVENT_LOAD_OPTION);
            AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "--- INFO. $mLoadOption.run()");
        }

    };

    // Asynchronously Load Frequency table
    private Runnable mLoadFrequencyTable = new Runnable() {

        @Override
        public void run() {
            AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "+++ INFO. $mLoadFrequencyTable.run()");

            Log.d(TAG, "mLoadFrequencyTable > start ");

            if(isJapaneseType()) {
                try {
                    mFreqTable = getAsReaderP3xU().getLbt();
//                    int i = -1;
                    for (AsReaderP3xULbtItem item : mFreqTable) {
//                        i ++;
                        if (item.isUsed() == true)
                        {
                            mUserConfigs.setFrequency(item.getFrequency());
                            mUserConfigs.Save();
                        }
                    }
                } catch (AsReaderP3xUException e) {
                    AsReaderP3xUASRLog.e(TAG, e, "ERROR. $mLoadFrequencyTable.run() - Failed to get LBT table");
                }
            }

            mEvent.post(new Runnable() {

                @Override
                public void run() {
                    WaitDialog.hide();
                    showFrequencyDialog();
                }

            });
            mEvent.sendEmptyMessage(EVENT_LOAD_OPTION);
            AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "--- INFO. $mLoadOption.run()");
        }

    };

    // Asynchronous Save Option
    private Runnable mSaveOption = new Runnable() {

        @Override
        public void run() {
            AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "+++ INFO. $mSaveOption.run()");

            SharedPreferences prefs = getSharedPreferences(APP_NAME, MODE_PRIVATE);
            SharedPreferences.Editor editor = prefs.edit();
            editor.apply();
            // Set Buzzer
            try {
                Thread.sleep(100);
                getAsReaderP3xU().setBuzzer(mBuzzer); // 写：7e776231 ~wb1  // 回：~Wb0000  0  000FK0011
                mUserConfigs.setVolume(mBuzzer);
                mUserConfigs.Save();
            } catch (AsReaderP3xUException | InterruptedException e) {
                AsReaderP3xUASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set buzzer (%s)", mBuzzer);
            }
            if (deviceModel == DeviceModel.P3xU) {
                // Set Idle Time
                try {
                    getAsReaderP3xU().setIdleTime(mIdleTime);
                    mUserConfigs.setIdleTime(mIdleTime);
                    mUserConfigs.Save();
                } catch (AsReaderP3xUException e) {
                    AsReaderP3xUASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set idle time (%d)", mIdleTime);
                }
                // Set Inventory Time
                try {
                    getAsReaderP3xU().setInventoryTime(mInventoryTime);
                    mUserConfigs.setInventoryTime(mInventoryTime);
                    mUserConfigs.Save();
                } catch (AsReaderP3xUException e) {
                    AsReaderP3xUASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set inventory time (%d)", mInventoryTime);
                }
            }
            // Set Q Value
            try {
                Thread.sleep(100);
                getAsReaderP3xU().setQValue(mQValue); // 写 7e7751360d0a ~wQ6 // 回：~WQ0000  0  000FK0011
                mUserConfigs.setQValue(mQValue);
                mUserConfigs.Save();
            } catch (AsReaderP3xUException | InterruptedException e) {
                AsReaderP3xUASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set Q value (%d)", mQValue);
            }

            mEvent.post(new Runnable() {

                @Override
                public void run() {
                    WaitDialog.hide();

                    mIsSaved = true;

                    String messageTitle = "Saved successfully";

                    AlertDialog.Builder builder = new AlertDialog.Builder(OptionActivity.this);
                    builder.setTitle("Success");
                    builder.setMessage(messageTitle)
                            .setCancelable(false)
                            .setPositiveButton("OK", null);

                    AlertDialog dlg = builder.create();
                    dlg.show();
                    enableActionWidgets(true);

                }

            });
            AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "+++ INFO. $mSaveOption.run()");
        }

    };

    private void mDefaultOption() {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "+++ INFO.mDefaultOption()");
        AsReaderP3xUResultCode res = getAsReaderP3xU().defaultParameter();
        if (res != AsReaderP3xUResultCode.NoError) {
            mIsDefault = false;
            WaitDialog.hide();
            String messageTitle = "Set to default failed";
            AlertDialog.Builder builder = new AlertDialog.Builder(OptionActivity.this);
            builder.setTitle("Fail");
            builder.setMessage(messageTitle)
                    .setCancelable(false)
                    .setPositiveButton("OK", null);

            AlertDialog dlg = builder.create();
            dlg.show();
            enableActionWidgets(true);
        } else {
            mIsDefault = true;
            mIsSaved = true;
            new Thread(mLoadOption).start();
        }
    };

    // Initialize Reader
    @Override
    protected void initAsReaderP3xU() {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initAsReaderP3xU()");
    }

    // Initialize Widgets
    protected void initWidgets() {
        txtBuzzer = (TextView) findViewById(R.id.buzzer);
        txtBuzzer.setOnClickListener(this);
        txtInventoryTime = (TextView) findViewById(R.id.inventory_time);
        txtInventoryTime.setOnClickListener(this);
        txtIdleTime = (TextView) findViewById(R.id.idle_time);
        txtIdleTime.setOnClickListener(this);
        txtQValue = (TextView) findViewById(R.id.q_value);
        txtQValue.setOnClickListener(this);
        txtFrequencyAutomatic = (TextView) findViewById(R.id.frequency_automatic);
        txtFrequencyAutomatic.setOnClickListener(this);
        btnLbtChannel = (Button) findViewById(R.id.btn_lbt_channel);
        btnLbtChannel.setOnClickListener(this);

        btnSave = (Button) findViewById(R.id.save);
        btnSave.setOnClickListener(this);
        btnDefault = (Button) findViewById(R.id.option_default);
        btnDefault.setOnClickListener(this);
        layoutLBT = (LinearLayout) findViewById(R.id.lay_lbt);
        layoutInventoryTime = (LinearLayout) findViewById(R.id.lay_inventoryTime);
        layoutIdleTime = (LinearLayout) findViewById(R.id.lay_idleTime);

        layoutFrequency = (LinearLayout) findViewById(R.id.layout_frequency);
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initWidgets()");
    }

    // Enable/Disable Action Widgets
    protected void enableActionWidgets(boolean enabled) {
        super.enableActionWidgets(enabled);
        mIsEnabled = enabled;
        btnSave.setEnabled(enabled);
        btnDefault.setEnabled(enabled);

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
    }

    private void displayBuzzer() {
        txtBuzzer.setText(mBuzzer.toString());
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. displayBuzzer() - [%s]", mBuzzer);
    }

    private void displayInventoryTime() {
        txtInventoryTime.setText(String.format(Locale.US, "%d", mInventoryTime));
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. displayInventoryTime() - [%d]", mInventoryTime);
    }

    private void displayIdleTime() {
        txtIdleTime.setText(String.format(Locale.US, "%d", mIdleTime));
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. displayIdleTime() - [%d]", mIdleTime);
    }

    private void displayQValue() {
        txtQValue.setText(String.format(Locale.US, "%d", mQValue));
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. displayQValue() - [%s]", mQValue);
    }

    private void displayFrequencyTable() {
        txtFrequencyAutomatic.setText(mFrequencyAutomatic ? "On" : "Off");
        if (isJapaneseType() && !mFrequencyAutomatic)
            layoutLBT.setVisibility(View.VISIBLE);
        else
            layoutLBT.setVisibility(View.GONE);
    }

    private boolean isJapaneseType() {
        return mGlobalBandType.contains("Japan");
    }

    private void showBuzzerDialog() {
        if (!mIsEnabled)
            return;

        LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
        final ListView value = (ListView) root.findViewById(R.id.list);
        final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
        for (AsReaderP3xUBuzzerState item : AsReaderP3xUBuzzerState.values()) {
            adapter.addItem(item.getCode(), item.toString());
        }
        value.setAdapter(adapter);
        value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.buzzer);
        builder.setView(root);
        builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                int position = value.getCheckedItemPosition();
                mBuzzer = AsReaderP3xUBuzzerState.valueOf(adapter.getValue(position));
                displayBuzzer();
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showBuzzerDialog().$PositiveButton.onClick()");
            }
        });
        builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showBuzzerDialog().$NegativeButton.onClick()");
            }
        });
        builder.setCancelable(true);
        builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

            @Override
            public void onCancel(DialogInterface dialog) {
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showBuzzerDialog().onCancel()");
            }
        });
        AlertDialog dialog = builder.create();
        dialog.setOnShowListener(new DialogInterface.OnShowListener() {

            @Override
            public void onShow(DialogInterface dialog) {
                int position = adapter.getPosition(mBuzzer.getCode());
                value.setItemChecked(position, true);
                value.setSelectionFromTop(position, 0);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showBuzzerDialog().onShow()");
            }
        });
        dialog.show();
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showBuzzerDialog()");
    }

    private void showInventoryTimeDialog() {
        if (!mIsEnabled)
            return;

        LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_unit, null);
        final EditText value = (EditText) root.findViewById(R.id.value);
        final TextView unit = (TextView) root.findViewById(R.id.unit);
        unit.setText(R.string.unit_ten_ms);
//        InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(10, 40000) };

//        value.setFilters(filters);

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.inventory_time);
        builder.setView(root);
        builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                int nTxOnTime = 0;
                try {
                    nTxOnTime = Integer.parseInt(value.getText().toString());
                } catch (Exception e) {
                    AsReaderP3xUASRLog.e(TAG, "ERROR. showInventoryTimeDialog().$PositiveButton.onClick() - parseInt [%s]", value.getText().toString());
                }
                if (nTxOnTime < 10) {
                    nTxOnTime = 10;
                } else if (nTxOnTime > 40000) {
                    nTxOnTime = 40000;
                }
                mInventoryTime = nTxOnTime;
                displayInventoryTime();
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if(imm != null)
                    imm.hideSoftInputFromWindow(value.getWindowToken(), 0);

                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showInventoryTimeDialog().$PositiveButton.onClick()");
            }
        });
        builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if(imm != null)
                    imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showInventoryTimeDialog().$NegativeButton.onClick()");
            }
        });
        builder.setCancelable(true);
        builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

            @Override
            public void onCancel(DialogInterface dialog) {
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if(imm != null)
                    imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showInventoryTimeDialog().onCancel()");
            }
        });
        AlertDialog dialog = builder.create();
        dialog.setOnShowListener(new DialogInterface.OnShowListener() {

            @Override
            public void onShow(DialogInterface dialog) {
                value.setText(String.format(Locale.US, "%d", mInventoryTime));
                value.selectAll();
                value.requestFocus();
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if(imm != null)
                    imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showInventoryTimeDialog().onShow()");
            }
        });
        dialog.show();
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showInventoryTimeDialog()");
    }

    private void showIdleTimeDialog() {
        if (!mIsEnabled)
            return;

        LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_unit, null);
        final EditText value = (EditText) root.findViewById(R.id.value);
        final TextView unit = (TextView) root.findViewById(R.id.unit);
        unit.setText(R.string.unit_ten_ms);
//        InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(0, 40000) };

//        value.setFilters(filters);

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.idle_time);
        builder.setView(root);
        builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                //int idleTime = 0;
                int txOffTime = 0;
                try {
                    txOffTime = Integer.parseInt(value.getText().toString());
                } catch (Exception e) {
                    AsReaderP3xUASRLog.e(TAG, "ERROR. showIdleTimeDialog().$PositiveButton.onClick() - parseInt(%s)", value.getText().toString());
                }
                if (txOffTime < 0) {
                    txOffTime = 0;
                } else if (txOffTime > 40000) {
                    txOffTime = 40000;
                }
                mIdleTime = txOffTime;
                displayIdleTime();
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if(imm != null)
                    imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showIdleTimeDialog().$PositiveButton.onClick()");
            }
        });
        builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if(imm != null)
                    imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showIdleTimeDialog().$NegativeButton.onClick()");
            }
        });
        builder.setCancelable(true);
        builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

            @Override
            public void onCancel(DialogInterface dialog) {
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if(imm != null)
                    imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showIdleTimeDialog().onCancel()");
            }
        });
        AlertDialog dialog = builder.create();
        dialog.setOnShowListener(new DialogInterface.OnShowListener() {

            @Override
            public void onShow(DialogInterface dialog) {
                value.setText(String.format(Locale.US, "%d", mIdleTime));
                value.selectAll();
                value.requestFocus();
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if(imm != null)
                    imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showIdleTimeDialog().onShow()");
            }
        });
        dialog.show();
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showIdleTimeDialog()");
    }

    private void showQValueDialog() {
        if (!mIsEnabled)
            return;

        LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
        final ListView value = (ListView) root.findViewById(R.id.list);
        final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);

        for (int n = qvalueMin; n <= qvalueMax; n++) {
            adapter.addItem(n, String.format(Locale.US, "%02d", n));
        }
        value.setAdapter(adapter);
        value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.q_value);
        builder.setView(root);
        builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                int position = -1;
                try {
                    position = value.getCheckedItemPosition();
                    mQValue = adapter.getValue(position);

                } catch (Exception e) {
                    AsReaderP3xUASRLog.e(TAG, "ERROR. showQValueDialog().$PositiveButton.onClick() - position(%d)", position);
                }
                displayQValue();
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQValueDialog().$PositiveButton.onClick()");
            }
        });
        builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQValueDialog().$NegativeButton.onClick()");
            }
        });
        builder.setCancelable(true);
        builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

            @Override
            public void onCancel(DialogInterface dialog) {
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQValueDialog().onCancel()");
            }
        });
        AlertDialog dialog = builder.create();
        dialog.setOnShowListener(new DialogInterface.OnShowListener() {

            @Override
            public void onShow(DialogInterface dialog) {
                int position = adapter.getPosition(mQValue);
                value.setItemChecked(position, true);
                value.setSelectionFromTop(position, 0);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQValueDialog().onShow()");
            }
        });
        dialog.show();
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQValueDialog()");
    }

    private void showFrequencyAutomaticDialog() {
        LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
        final ListView value = (ListView) root.findViewById(R.id.list);
        final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
        adapter.addItem(0, "Off");
        adapter.addItem(1, "On");
        value.setAdapter(adapter);
        value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.frequency_automatic);
        builder.setView(root);
        builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                try {
                    int position = value.getCheckedItemPosition();
                    mFrequencyAutomatic = adapter.getValue(position) > 0;
                    getAsReaderP3xU().setFrequencyAutomatic(mFrequencyAutomatic);
                    mUserConfigs.setFrequencyAuto(mFrequencyAutomatic);
                    mUserConfigs.Save();
                    // [systemSetting setFrequencyAutomatic:_lbtSwitch.isOn];
                } catch (Exception e) {
                    AsReaderP3xUASRLog.e(TAG, "ERROR.  showFrequencyAutomaticDialog().$PositiveButton.onClick() - [%s]"
                            , e.getMessage());
                }
                displayFrequencyTable();
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showFrequencyAutomaticDialog().$PositiveButton.onClick()");
            }
        });
        builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showFrequencyAutomaticDialog().$NegativeButton.onClick()");
            }
        });
        builder.setCancelable(true);
        builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

            @Override
            public void onCancel(DialogInterface dialog) {
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showFrequencyAutomaticDialog().onCancel()");
            }
        });
        AlertDialog dialog = builder.create();
        dialog.setOnShowListener(new DialogInterface.OnShowListener() {

            @Override
            public void onShow(DialogInterface dialog) {
                int position = adapter.getPosition(mFrequencyAutomatic ? 1 : 0);
                value.setItemChecked(position, true);
                value.setSelectionFromTop(position, 0);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showFrequencyAutomaticDialog().onShow()");
            }
        });
        dialog.show();
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showFrequencyAutomaticDialog()");
    }

    private void showFrequencyDialog() {
        final LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
        final ListView table = (ListView) root.findViewById(R.id.list);
        final LbtAdapter adapter = new LbtAdapter(this, android.R.layout.simple_list_item_single_choice);

        int i = 0;
        for (AsReaderP3xULbtItem item : mFreqTable) {
            adapter.addItem(item.getSlot(), item.getFrequency(), item.isUsed());
            if (item.isUsed() == true)
            {
                showFrequency = i;
            }
            i ++;
        }
        table.setAdapter(adapter);
        table.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.lbt_channel_title);
        builder.setView(root);
        builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                int position = 0;
                try {
                    position = table.getCheckedItemPosition();
                    showFrequency = adapter.getValue(position);
                    for (int i = 0; i < mFreqTable.length; i++) {
                        AsReaderP3xULbtItem item = mFreqTable[i];
                        if (position == i) {
                            item.setUsed(true);
                        }else{
                            item.setUsed(false);
                        }
                    }
                    getAsReaderP3xU().setLbt(mFreqTable);
                    AsReaderP3xULbtItem item = mFreqTable[position];
                    mUserConfigs.setFrequency(item.getFrequency());
                    mUserConfigs.Save();
                } catch (Exception e) {
                    AsReaderP3xUASRLog.e(TAG, "ERROR.  showFrequencyAutomaticDialog().$PositiveButton.onClick() - [%s]"
                            , e.getMessage());
                }
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showLbtChannelDialog().$PositiveButton.onClick()");
            }
        });
        builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showLbtChannelDialog().$NegativeButton.onClick()");
            }
        });
        builder.setCancelable(true);
        builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

            @Override
            public void onCancel(DialogInterface dialog) {
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showLbtChannelDialog().onCancel()");
            }
        });
        AlertDialog dialog = builder.create();
        dialog.setOnShowListener(new DialogInterface.OnShowListener() {

            @Override
            public void onShow(DialogInterface dialog) {
                int position = adapter.getPosition(showFrequency);
                table.setItemChecked(position, true);
                table.setSelectionFromTop(position, 0);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showLbtChannelDialog().onShow()");
            }
        });
        dialog.show();
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showLbtChannelDialog()");
    }
}
