package com.asreader.p3xu.view;

import com.asreader.p3xu.AsReaderP3xU;
import com.asreader.p3xu.rfid.exception.AsReaderP3xUException;
import com.asreader.p3xu.rfid.params.AsReaderP3xULockParam;
import com.asreader.p3xu.rfid.type.AsReaderP3xUActionState;
import com.asreader.p3xu.rfid.type.AsReaderP3xUMemoryBank;
import com.asreader.p3xu.rfid.type.AsReaderP3xUResultCode;

import java.lang.ref.WeakReference;
import java.util.Locale;

import com.asreader.R;
import com.asreader.p3xu.diagnostics.AsReaderP3xUASRLog;
import com.asreader.support.andro.app.rfid.dialog.IDialogResultListener;
import com.asreader.support.andro.app.rfid.dialog.PasswordDialog;
import com.asreader.support.andro.app.rfid.dialog.WaitDialog;
import com.asreader.support.andro.app.rfid.util.ResUtil;
import com.asreader.support.andro.app.rfid.view.base.AccessActivity;

import android.app.ActionBar;
import android.content.DialogInterface;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Toast;

public class TagAccessActivity extends AccessActivity {

	private static final String TAG = TagAccessActivity.class.getSimpleName();

	private static final int KILL_PASSWORD = 0;
	private static final int ACCESS_PASSWORD = 1;
	private static final int EPC = 2;
	private static final int TID = 3;
	private static final int USER = 4;
	private static final int MAX_ACCESS = 5;

	private static final int SUBACTION_IDLE = 0;
	private static final int SUBACTION_LOCK = 1;
	private static final int SUBACTION_UNLOCK = 2;
	private static final int SUBACTION_PERMALOCK = 3;
	private static final int SUBACTION_KILL = 4;
	private static final int SUBACTION_SET_ACCESS_PWD = 5;
	private static final int SUBACTION_SET_KILL_PWD = 6;

	private CheckBox[] chkAccess;
	private Button btnLock;
	private Button btnUnlock;
	private Button btnPermaLock;
	private Button btnKill;
	private Button btnSetAccessPassword;
	private Button btnSetKillPassword;


	private int mSubAction = SUBACTION_IDLE;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_tag_access);
		ActionBar actionBar = getActionBar();
		if (actionBar != null) {
			actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
			actionBar.setCustomView(R.layout.activity_title);
			actionBar.setDisplayHomeAsUpEnabled(true);
			actionBar.setDisplayShowHomeEnabled(true);
		}
		TextView titleText = (TextView)findViewById(R.id.title_text);
		titleText.setText(R.string.lock_memory_name);
		final int version = Build.VERSION.SDK_INT;
		if(version >= Build.VERSION_CODES.M)
			titleText.setTextColor(getColor(getApplicationContext(), R.color.black));

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		createAsReaderP3xU();

		// Initialize Widget
		initWidgets();

		// Initialize Reader
		initAsReaderP3xU();

		InitTask task = new InitTask(this);
		task.execute();
//		// Initialize Mask
//		initMask();
//
//		resetActionResult();
//
//		setMask(getMask());
//
//		// Enable action widget
//		enableActionWidgets(true);
//
//		outputMessage(R.string.access_intro_message);
//		mSubAction = SUBACTION_IDLE;
	}

	@Override
	protected void onDestroy() {

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		destroyAsReaderP3xU();

		super.onDestroy();
	}

	@Override
	protected void onStart() {
		super.onStart();

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onStart()");
	}

	@Override
	protected void onStop() {
		
		if(getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Lock || getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Unlock ||
				getAsReaderP3xU().getAction() == AsReaderP3xUActionState.PermaLock || getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Kill)
			getAsReaderP3xU().stop();
		
		// Rollback Mask
		exitMask();

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onStart()");
		super.onStop();
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onResume()");
	}

	@Override
	protected void onPause() {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onPause()");
		super.onPause();
	}

	@Override
	public void onClick(View v) {

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));

		switch (v.getId()) {
		case R.id.action_lock:
			if (getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop) {
				lockTag();
			} else {
				mSubAction = SUBACTION_IDLE;
				stopAction();
			}
			break;
		case R.id.action_unlock:
			if (getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop) {
				unlockTag();
			} else {
				mSubAction = SUBACTION_IDLE;
				stopAction();
			}
			break;
		case R.id.action_perma_lock:
			if (getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop) {
				permaLockTag();
			} else {
				mSubAction = SUBACTION_IDLE;
				stopAction();
			}
			break;
		case R.id.action_kill:
			if (getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop) {
				killTag();
			} else {
				mSubAction = SUBACTION_IDLE;
				stopAction();
			}
			break;
		case R.id.action_access_password:
			if (getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop) {
				setAccessPassword();
			} else {
				mSubAction = SUBACTION_IDLE;
				stopAction();
			}
			break;
		case R.id.action_kill_password:
			if (getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop) {
				setKillPassword();
			} else {
				mSubAction = SUBACTION_IDLE;
				stopAction();
			}
			break;
		}
		super.onClick(v);
	}

	@Override
	public void onActionChanged(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action) {
		AsReaderP3xUASRLog.d(TAG, AsReaderP3xUASRLog.DEBUG, "EVENT. onActionChanged(%s)", action);

		enableActionWidgets(true);

		if (action == AsReaderP3xUActionState.Stop) {
			mSubAction = SUBACTION_IDLE;
			if (!isActionResult()) {
				outputMessage(R.string.access_intro_message);
			}
		}
	}

	@Override
	public void onAccessResult(AsReaderP3xU asReaderP3xU, AsReaderP3xUResultCode code, AsReaderP3xUActionState action, String epc, String data) {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onAccessResult(%s, %s, [%s], [%s])", code, action, epc == null ? "" : epc,
				data == null ? "" : data);

		setActionResult();

		if (code != AsReaderP3xUResultCode.NoError) {
			outputFailMessage(code.toString());
			return;
		}

		if(epc != null)
			outputSelection(epc.length() > 4 ? epc.substring(4) : epc);

		outputSuccessMessage(R.string.access_success);
		mSubAction = SUBACTION_IDLE;
	}

	// Lock Tag
	private void lockTag() {
		mSubAction = SUBACTION_LOCK;
		clearWidgets();
		enableActionWidgets(false);
		outputSelection("");
		outputMessage(R.string.access_lock_message);

		resetActionResult();

		AsReaderP3xULockParam param = new AsReaderP3xULockParam(chkAccess[KILL_PASSWORD].isChecked(), chkAccess[ACCESS_PASSWORD].isChecked(),
				chkAccess[EPC].isChecked(), chkAccess[TID].isChecked(), chkAccess[USER].isChecked());

		String password = getAccessPassword();
		try {
			Thread.sleep(50);
			getAsReaderP3xU().setAccessPassword(password);
		} catch (AsReaderP3xUException | InterruptedException e) {
			AsReaderP3xUASRLog.e(TAG, "ERROR. lockTag() - Failed to set password [%s]", password);
		}

		int time = getOperationTime();
		try {
			Thread.sleep(50);
			getAsReaderP3xU().setOperationTime(time);
		} catch (AsReaderP3xUException | InterruptedException e) {
			AsReaderP3xUASRLog.e(TAG, e, "ERROR. lockTag() - Failed to set operation time [%d]", time);
		}

		AsReaderP3xUResultCode res;
		if ((res = getAsReaderP3xU().lock(param)) != AsReaderP3xUResultCode.NoError) {
			AsReaderP3xUASRLog.e(TAG, "ERROR. lockTag() - Failed to start lock tag [%s]", param);
			String msg = String.format(Locale.US, "%s : 0x%04X", res.toString(), res.getCode());
			Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();
			enableActionWidgets(true);
			outputFailMessage(R.string.access_fail_lock_message);
			return;
		}
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. lockTag()");
	}

	// Unlock Tag
	private void unlockTag() {
		mSubAction = SUBACTION_UNLOCK;
		clearWidgets();
		enableActionWidgets(false);
		outputSelection("");
		outputMessage(R.string.access_unlock_message);

		resetActionResult();

		AsReaderP3xULockParam param = new AsReaderP3xULockParam(chkAccess[KILL_PASSWORD].isChecked(), chkAccess[ACCESS_PASSWORD].isChecked(),
				chkAccess[EPC].isChecked(), chkAccess[TID].isChecked(), chkAccess[USER].isChecked());

		String password = getAccessPassword();
		try {
			Thread.sleep(50);
			getAsReaderP3xU().setAccessPassword(password);
		} catch (AsReaderP3xUException | InterruptedException e) {
			AsReaderP3xUASRLog.e(TAG, e, "ERROR. unlockTag() - Failed to set password [%s]", password);
		}

		int time = getOperationTime();
		try {
			Thread.sleep(50);
			getAsReaderP3xU().setOperationTime(time);
		} catch (AsReaderP3xUException | InterruptedException e) {
			AsReaderP3xUASRLog.e(TAG, e, "ERROR. unlockTag() - Failed to set operation time [%d]", time);
		}

		AsReaderP3xUResultCode res;
		if ((res = getAsReaderP3xU().unlock(param)) != AsReaderP3xUResultCode.NoError) {
			AsReaderP3xUASRLog.e(TAG, "ERROR. unlockTag() - Failed to start lock tag [%s]", param);
			String msg = String.format(Locale.US, "%s : 0x%04X", res.toString(), res.getCode());
			Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();
			enableActionWidgets(true);
			outputFailMessage(R.string.access_fail_unlock_message);
			return;
		}
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. unlockTag()");
	}

	// PermaLock Tag
	private void permaLockTag() {
		mSubAction = SUBACTION_PERMALOCK;
		clearWidgets();
		enableActionWidgets(false);
		outputSelection("");
		outputMessage(R.string.access_perma_lock_message);

		resetActionResult();

		AsReaderP3xULockParam param = new AsReaderP3xULockParam(chkAccess[KILL_PASSWORD].isChecked(), chkAccess[ACCESS_PASSWORD].isChecked(),
				chkAccess[EPC].isChecked(), chkAccess[TID].isChecked(), chkAccess[USER].isChecked());

		String password = getAccessPassword();
		try {
			Thread.sleep(50);
			getAsReaderP3xU().setAccessPassword(password);
		} catch (AsReaderP3xUException | InterruptedException e) {
			AsReaderP3xUASRLog.e(TAG, "ERROR. permaLockTag() - Failed to set password [%s]", password);
		}

		int time = getOperationTime();
		try {
			Thread.sleep(50);
			getAsReaderP3xU().setOperationTime(time);
		} catch (AsReaderP3xUException | InterruptedException e) {
			AsReaderP3xUASRLog.e(TAG, e, "ERROR. permaLockTag() - Failed to set operation time [%d]", time);
		}

		AsReaderP3xUResultCode res;
		if ((res = getAsReaderP3xU().permaLock(param)) != AsReaderP3xUResultCode.NoError) {
			AsReaderP3xUASRLog.e(TAG, "ERROR. permaLockTag() - Failed to start lock tag [%s]", param);
			String msg = String.format(Locale.US, "%s : 0x%04X", res.toString(), res.getCode());
			Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();
			enableActionWidgets(true);
			outputFailMessage(R.string.access_fail_perma_lock_message);
			return;
		}
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. permaLockTag()");
	}

	// Kill Tag
	private void killTag() {

		enableActionWidgets(false);
		final PasswordDialog dlg = new PasswordDialog(this, R.string.kill_password);
		dlg.setResultListener(new IDialogResultListener() {

			@Override
			public void onOkClick(int what, DialogInterface dialog) {
				mSubAction = SUBACTION_KILL;
				String password = dlg.getPassword();

				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. killTag().onOkClick() - [%s]", password);

				clearWidgets();
				outputSelection("");
				outputMessage(R.string.access_kill_message);

				resetActionResult();

				int time = getOperationTime();

				try {
					Thread.sleep(50);
					getAsReaderP3xU().setOperationTime(time);
				} catch (AsReaderP3xUException | InterruptedException e) {
					AsReaderP3xUASRLog.e(TAG, e, "ERROR. killTag().onOkClick() - Failed to set operation time [%d]", time);
				}

				AsReaderP3xUResultCode res;
				if ((res = getAsReaderP3xU().kill(password)) != AsReaderP3xUResultCode.NoError) {
					AsReaderP3xUASRLog.e(TAG, "ERROR. killTag().onOkClick() - Failed to start kill tag [%s]", password);
					String msg = String.format(Locale.US, "%s : 0x%04X", res.toString(), res.getCode());
					Toast.makeText(TagAccessActivity.this, msg, Toast.LENGTH_SHORT).show();
					enableActionWidgets(true);
					outputFailMessage(R.string.access_fail_kill_message);
				}
			}

			@Override
			public void onCancelClick(int what, DialogInterface dialog) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. killTag().onCancelClick()");
				enableActionWidgets(true);
			}

		});
		dlg.show();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. killTag()");
	}

	// Set Access Password
	private void setAccessPassword() {
		enableActionWidgets(false);
		final PasswordDialog dlg = new PasswordDialog(this, R.string.set_access_password_title);
		dlg.setResultListener(new IDialogResultListener() {

			@Override
			public void onOkClick(int what, DialogInterface dialog) {
				mSubAction = SUBACTION_SET_ACCESS_PWD;
				String password = dlg.getPassword();
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setAccessPassword().onOkClick() - [%s]", password);

				clearWidgets();
				outputSelection("");
				outputMessage(R.string.access_set_access_password_message);

				resetActionResult();

				String accessPwd = getAccessPassword();
				try {
					Thread.sleep(50);
					getAsReaderP3xU().setAccessPassword(accessPwd);
				} catch (AsReaderP3xUException | InterruptedException e) {
					AsReaderP3xUASRLog.e(TAG, "ERROR. setAccessPassword().onOkClick() - Failed to set password [%s]", accessPwd);
				}

				int time = getOperationTime();

				try {
					Thread.sleep(50);
					getAsReaderP3xU().setOperationTime(time);
				} catch (AsReaderP3xUException | InterruptedException e) {
					AsReaderP3xUASRLog.e(TAG, e, "ERROR. setAccessPassword().onOkClick() - Failed to set operation time [%d]", time);
				}

				if (getAsReaderP3xU().writeMemory(AsReaderP3xUMemoryBank.Reserved, 2, password) != AsReaderP3xUResultCode.NoError) {
					AsReaderP3xUASRLog.e(TAG, "ERROR. setAccessPassword().onOkClick() - Failed to start write access password [%s]",
							password);
					enableActionWidgets(true);
					outputFailMessage(R.string.access_fail_set_access_password_message);
				}
			}

			@Override
			public void onCancelClick(int what, DialogInterface dialog) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setAccessPassword().onCancelClick()");
				enableActionWidgets(true);
			}

		});
		dlg.show();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setAccessPassword()");
	}

	// Set Kill Password
	private void setKillPassword() {
		enableActionWidgets(false);
		final PasswordDialog dlg = new PasswordDialog(this, R.string.set_kill_password_title);
		dlg.setResultListener(new IDialogResultListener() {

			@Override
			public void onOkClick(int what, DialogInterface dialog) {
				mSubAction = SUBACTION_SET_KILL_PWD;
				String password = dlg.getPassword();
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setKillPassword().onOkClick() - [%s]", password);

				clearWidgets();
				outputSelection("");
				outputMessage(R.string.access_set_kill_password_message);

				resetActionResult();

				String accessPwd = getAccessPassword();
				try {
					Thread.sleep(50);
					getAsReaderP3xU().setAccessPassword(accessPwd);
				} catch (AsReaderP3xUException | InterruptedException e) {
					AsReaderP3xUASRLog.e(TAG, e, "ERROR. setKillPassword() - Failed to set password [%s]", accessPwd);
				}

				int time = getOperationTime();

				try {
					Thread.sleep(50);
					getAsReaderP3xU().setOperationTime(time);
				} catch (AsReaderP3xUException | InterruptedException e) {
					AsReaderP3xUASRLog.e(TAG, e, "ERROR. setKillPassword() - Failed to set operation time [%d]", time);
				}

				if (getAsReaderP3xU().writeMemory(AsReaderP3xUMemoryBank.Reserved, 0, password) != AsReaderP3xUResultCode.NoError) {
					AsReaderP3xUASRLog.e(TAG, "ERROR. setKillPassword() - Failed to start write kill password [%s]", password);
					enableActionWidgets(true);
					outputFailMessage(R.string.access_fail_set_kill_password_message);
				}
			}

			@Override
			public void onCancelClick(int what, DialogInterface dialog) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setKillPassword().onCancelClick()");
				enableActionWidgets(true);
			}

		});
		dlg.show();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setKillPassword()");
	}

	// Initialize Reader
	@Override
	protected void initAsReaderP3xU() {
		super.initAsReaderP3xU();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initAsReaderP3xU()");
	}

	// Initialize Widgets
	protected void initWidgets() {
		super.initWidgets();

		ScrollView scrlParams = (ScrollView) findViewById(R.id.lock_params);
		scrlParams.setBackgroundColor(getColor(getApplicationContext(), R.color.message_background));

		int[] ids = new int[] { R.id.kill_password, R.id.access_password, R.id.epc, R.id.tid, R.id.user };
		chkAccess = new CheckBox[MAX_ACCESS];

		for (int i = 0; i < MAX_ACCESS; i++) {
			chkAccess[i] = (CheckBox) findViewById(ids[i]);
		}
		// Lock Button
		btnLock = (Button) findViewById(R.id.action_lock);
		btnLock.setOnClickListener(this);
		// Unlock Button
		btnUnlock = (Button) findViewById(R.id.action_unlock);
		btnUnlock.setOnClickListener(this);
		// PermaLock Button
		btnPermaLock = (Button) findViewById(R.id.action_perma_lock);
		btnPermaLock.setOnClickListener(this);
		// Kill Button
		btnKill = (Button) findViewById(R.id.action_kill);
		btnKill.setOnClickListener(this);
		// Set Access Password Button
		btnSetAccessPassword = (Button) findViewById(R.id.action_access_password);
		btnSetAccessPassword.setOnClickListener(this);
		// Set Kill Password Button
		btnSetKillPassword = (Button) findViewById(R.id.action_kill_password);
		btnSetKillPassword.setOnClickListener(this);

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initWidgets()");
	}

	private boolean isEnabledWidget(boolean enabled, AsReaderP3xUActionState state, int subAction) {
		AsReaderP3xUActionState action = getAsReaderP3xU().getAction();
		return enabled ? (action == AsReaderP3xUActionState.Stop ? enabled
				: (action == state ? (mSubAction == subAction ? enabled : false) : false)) : enabled;
	}

	// Enable/Disable Action Widgets
	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);

		for (int i = 0; i < MAX_ACCESS; i++) {
			chkAccess[i].setEnabled(isEnabledWidget(enabled));
		}
		btnLock.setEnabled(isEnabledWidget(enabled, AsReaderP3xUActionState.Lock, SUBACTION_LOCK));
		btnUnlock.setEnabled(isEnabledWidget(enabled, AsReaderP3xUActionState.Unlock, SUBACTION_UNLOCK));
		btnPermaLock.setEnabled(isEnabledWidget(enabled, AsReaderP3xUActionState.PermaLock, SUBACTION_PERMALOCK));
		btnKill.setEnabled(isEnabledWidget(enabled, AsReaderP3xUActionState.Kill, SUBACTION_KILL));
		btnSetAccessPassword.setEnabled(isEnabledWidget(enabled, AsReaderP3xUActionState.WriteMemory, SUBACTION_SET_ACCESS_PWD));
		btnSetKillPassword.setEnabled(isEnabledWidget(enabled, AsReaderP3xUActionState.WriteMemory, SUBACTION_SET_KILL_PWD));
		if (enabled) {
			switch (getAsReaderP3xU().getAction()) {
			case Lock:
				switch (mSubAction) {
				case SUBACTION_LOCK:
					btnLock.setText(R.string.action_stop);
					btnUnlock.setText(R.string.action_unlock_tag);
					btnPermaLock.setText(R.string.action_perma_lock_tag);
					btnKill.setText(R.string.action_kill_tag);
					btnSetAccessPassword.setText(R.string.set_access_password);
					btnSetKillPassword.setText(R.string.set_kill_password);
					break;
				case SUBACTION_UNLOCK:
					btnLock.setText(R.string.action_lock_tag);
					btnUnlock.setText(R.string.action_stop);
					btnPermaLock.setText(R.string.action_perma_lock_tag);
					btnKill.setText(R.string.action_kill_tag);
					btnSetAccessPassword.setText(R.string.set_access_password);
					btnSetKillPassword.setText(R.string.set_kill_password);
					break;
				case SUBACTION_PERMALOCK:
					btnLock.setText(R.string.action_lock_tag);
					btnUnlock.setText(R.string.action_unlock_tag);
					btnPermaLock.setText(R.string.action_stop);
					btnKill.setText(R.string.action_kill_tag);
					btnSetAccessPassword.setText(R.string.set_access_password);
					btnSetKillPassword.setText(R.string.set_kill_password);
					break;
				}
				break;
			case Kill:
				if (mSubAction == SUBACTION_KILL) {
					btnLock.setText(R.string.action_lock_tag);
					btnUnlock.setText(R.string.action_unlock_tag);
					btnPermaLock.setText(R.string.action_perma_lock_tag);
					btnKill.setText(R.string.action_stop);
					btnSetAccessPassword.setText(R.string.set_access_password);
					btnSetKillPassword.setText(R.string.set_kill_password);
				}
				break;
			case WriteMemory:
				btnLock.setText(R.string.action_lock_tag);
				btnUnlock.setText(R.string.action_unlock_tag);
				btnPermaLock.setText(R.string.action_perma_lock_tag);
				btnKill.setText(R.string.action_kill_tag);
				switch (mSubAction) {
				case SUBACTION_SET_ACCESS_PWD:
					btnSetAccessPassword.setText(R.string.action_stop);
					btnSetKillPassword.setText(R.string.set_kill_password);
					break;
				case SUBACTION_SET_KILL_PWD:
					btnSetAccessPassword.setText(R.string.set_access_password);
					btnSetKillPassword.setText(R.string.action_stop);
					break;
				}
				break;
			case Stop:
				btnLock.setText(R.string.action_lock_tag);
				btnUnlock.setText(R.string.action_unlock_tag);
				btnPermaLock.setText(R.string.action_perma_lock_tag);
				btnKill.setText(R.string.action_kill_tag);
				btnSetAccessPassword.setText(R.string.set_access_password);
				btnSetKillPassword.setText(R.string.set_kill_password);
				break;
			case Inventory:
				break;
			case PermaLock:
				btnLock.setText(R.string.action_lock_tag);
				btnUnlock.setText(R.string.action_unlock_tag);
				btnPermaLock.setText(R.string.action_stop);
				btnKill.setText(R.string.action_kill_tag);
				btnSetAccessPassword.setText(R.string.set_access_password);
				btnSetKillPassword.setText(R.string.set_kill_password);
				break;
			case ReadMemory:
				break;
			case Unlock:
				btnLock.setText(R.string.action_lock_tag);
				btnUnlock.setText(R.string.action_stop);
				btnPermaLock.setText(R.string.action_perma_lock_tag);
				btnKill.setText(R.string.action_kill_tag);
				btnSetAccessPassword.setText(R.string.set_access_password);
				btnSetKillPassword.setText(R.string.set_kill_password);
				break;
			default:
				break;
			}
		}
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}

	private static class InitTask extends AsyncTask<Void, Void, Boolean> {
		private final WeakReference<TagAccessActivity> mActivity;

		InitTask(TagAccessActivity activity) {
			mActivity = new WeakReference<>(activity);
		}

		@Override
		protected void onPreExecute() {
			TagAccessActivity activity = mActivity.get();
			WaitDialog.show(activity, R.string.init_mask);
			super.onPreExecute();
		}

		@Override
		protected Boolean doInBackground(Void... params) {
			TagAccessActivity activity = mActivity.get();
			// Initialize Mask
			activity.initMask();
			
			return true;
		}
		
		@Override
		protected void onPostExecute(Boolean result) {
			TagAccessActivity activity = mActivity.get();

			activity.resetActionResult();

			activity.setMask(activity.getMask());

			// Enable action widget
			activity.enableActionWidgets(true);

			activity.outputMessage(R.string.access_intro_message);
			activity.mSubAction = SUBACTION_IDLE;
			WaitDialog.hide();
			super.onPostExecute(result);
		}
	}
}
