package com.asreader.support.andro.app.rfid.data;

import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.database.sqlite.SQLiteOpenHelper;

import java.util.ArrayList;

/**
 * cej - Added Class
 * DB - SQLite
 * zsj 2021-02-26 (code from 251G)
 * */

public class MySQLiteOpenHelper extends SQLiteOpenHelper {
    private static MySQLiteOpenHelper mInstance =  null;

    private static  final String DB_NAME    = "inventory.db";

    // RFID
    private static  final String TABLE_NAME = "inventory_tag";

    private static  final String FIELD_TAG      = "tag";
    private static  final String FIELD_COUNT      = "tagcount";
    private static  final String FIELD_TOTAL_COUNT = "totalcount";
    private static  final String FIELD_RSSI      = "rssi";
    private static  final String FIELD_TIME      = "time";

    private ArrayList<String> itemList = new ArrayList<>();

    private MySQLiteOpenHelper(Context context, String name,
                               CursorFactory factory, int version) {
        super(context, name, factory, version);

        SQLiteDatabase db  = this.getWritableDatabase();
        onCreate(db); // Process for creating tables again for deleted DB tables
    }

    public static MySQLiteOpenHelper getInstance(Context context)
    {
        if(mInstance == null)
            mInstance = new MySQLiteOpenHelper(context,DB_NAME,null,1);

        return mInstance;
    }


    // insert/delete/select RFID tag information to DB  ///////////////////////////////////////////////////////////////////
    public void insert(String tag, String tagcount, String rssi, long totalcount, String time) {

        SQLiteDatabase db  = this.getWritableDatabase();

        db.execSQL("INSERT INTO "+ TABLE_NAME+"("+FIELD_TAG+","+FIELD_COUNT+","+FIELD_RSSI+","+FIELD_TIME+","+FIELD_TOTAL_COUNT
                + ") VALUES('"+tag+"','"+tagcount+"','"+rssi+"','"+time+"',"+totalcount+");");
    }

    public void delete () {
        SQLiteDatabase db  = this.getWritableDatabase();
        db.execSQL("DELETE FROM "+TABLE_NAME);
        db.close();
    }

    public ArrayList select() {
        // 1) Reads data from db , 2) Saves results , 3) extracts and uses the data.
        SQLiteDatabase db  = this.getReadableDatabase(); // Obtain db objects. Read-only

        itemList.clear();
        String result = "";

        Cursor cursor = db.rawQuery("SELECT * FROM "+TABLE_NAME, null);
        while (cursor.moveToNext()){
            result = cursor.getString(cursor.getColumnIndex(FIELD_TAG))
                    + ","
                    + cursor.getString(cursor.getColumnIndex(FIELD_COUNT))
                    + ","
                    + cursor.getString(cursor.getColumnIndex(FIELD_RSSI))
                    + ","
                    + cursor.getString(cursor.getColumnIndex(FIELD_TIME))
                    + ","
                    + cursor.getString(cursor.getColumnIndex(FIELD_TOTAL_COUNT));

            itemList.add(result);
        }

        return itemList;
    }
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    // Barcode
    private static  final String TABLE_NAME_BARCODE = "readed_barcode";

    private static  final String FIELD_BARCODE      = "barcode";
    private static  final String FIELD_BARCODE_COUNT      = "barcode_count";
    private static  final String FIELD_BARCODE_TYPE      = "type";
    private static  final String FIELD_BARCODE_CODEID     = "code_id";
    private static  final String FIELD_BARCODE_TIME     = "barcode_time";

    // insert/delete/select BARCODE information to DB
    public void insertBarcode(String barcode, String count, String type, String code, String time) {
        SQLiteDatabase db  = this.getWritableDatabase();

        db.execSQL("INSERT INTO "+ TABLE_NAME_BARCODE+"("+FIELD_BARCODE+","+FIELD_BARCODE_COUNT+","
                +FIELD_BARCODE_TYPE+","+FIELD_BARCODE_CODEID+","+FIELD_BARCODE_TIME
                + ") VALUES('"+barcode+"','"+count+"','"+type+"','"+code+"','"+time+"');");
    }

    public void deleteBarcode () {
        SQLiteDatabase db  = this.getWritableDatabase();
        db.execSQL("DELETE FROM "+TABLE_NAME_BARCODE);
        db.close();
    }

    public ArrayList selectBarcode() {
        // 1) Reads data from db , 2) Saves results , 3) extracts and uses the data.
        SQLiteDatabase db  = this.getReadableDatabase(); // Obtain db objects. Read-only

        itemList.clear();
        String result = "";

        Cursor cursor = db.rawQuery("SELECT * FROM "+TABLE_NAME_BARCODE, null);
        while (cursor.moveToNext()){
            result = cursor.getString(cursor.getColumnIndex(FIELD_BARCODE))
                    + "<split>"
                    + cursor.getString(cursor.getColumnIndex(FIELD_BARCODE_COUNT))
                    + "<split>"
                    + cursor.getString(cursor.getColumnIndex(FIELD_BARCODE_TYPE))
                    + "<split>"
                    + cursor.getString(cursor.getColumnIndex(FIELD_BARCODE_CODEID))
                    + "<split>"
                    + cursor.getString(cursor.getColumnIndex(FIELD_BARCODE_TIME));

            itemList.add(result);
        }

        return itemList;
    }
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


    @Override
    public void onCreate(SQLiteDatabase db) {
        // When SQLiteOpenHelper first ran
        // RFID Table
        String sql = "CREATE TABLE IF NOT EXISTS "+TABLE_NAME+" (" +
                "_id integer primary key autoincrement, " +
                FIELD_TAG +" VARCHAR(255),"+FIELD_COUNT+" TEXT,"+FIELD_RSSI+" TEXT,"+FIELD_TIME+" LONG,"+FIELD_TOTAL_COUNT+" LONG);";
        db.execSQL(sql);

        // Barcode Table
        String sql2 = "CREATE TABLE IF NOT EXISTS "+TABLE_NAME_BARCODE+" (" +
                "_id integer primary key autoincrement, " +
                FIELD_BARCODE +" TEXT,"+FIELD_BARCODE_COUNT+" TEXT,"+FIELD_BARCODE_TYPE+" TEXT,"
                +FIELD_BARCODE_CODEID+" TEXT,"+FIELD_BARCODE_TIME+" LONG);";
        db.execSQL(sql2);
    }


    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        db.execSQL("DROP TABLE IF EXISTS "+TABLE_NAME);
        onCreate(db); // Process for creating tables again for deleted DB tables

        // Barcode
        db.execSQL("DROP TABLE IF EXISTS "+TABLE_NAME_BARCODE);
        onCreate(db);
    }

}