package com.asreader.support.andro.app.rfid.view.sub;

import com.asreader.p3xu.AsReaderP3xU;
import com.asreader.p3xu.device.type.AsReaderP3xUConnectionState;
import com.asreader.p3xu.device.type.AsReaderP3xUKeyState;
import com.asreader.p3xu.device.type.AsReaderP3xUKeyType;
import com.asreader.p3xu.diagnostics.AsReaderP3xUASRLog;
import com.asreader.p3xu.rfid.event.AsReaderP3xUEventListener;
import com.asreader.p3xu.rfid.exception.AsReaderP3xUException;
import com.asreader.p3xu.rfid.params.AsReaderP3xUSelectMaskEpcParam;
import com.asreader.p3xu.rfid.type.AsReaderP3xUActionState;
import com.asreader.p3xu.rfid.type.AsReaderP3xUCommandType;
import com.asreader.p3xu.rfid.type.AsReaderP3xUResultCode;
import com.asreader.R;
import com.asreader.p3xu.AsReaderP3xUManager;
import com.asreader.support.andro.app.rfid.dialog.WaitDialog;
import com.asreader.support.andro.app.rfid.dialog.WheelDialog.MaskWheelDialog;
import com.asreader.support.andro.app.rfid.util.ResUtil;
import com.asreader.support.andro.app.rfid.util.UserConfigs;
import com.wx.wheelview.widget.WheelViewDialog;

import android.annotation.SuppressLint;
import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.text.InputType;
import android.text.TextUtils;
import android.text.method.DigitsKeyListener;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class EpcMaskActivity extends Activity implements AsReaderP3xUEventListener, OnClickListener,
        OnItemClickListener, android.content.DialogInterface.OnClickListener {

    private static final String TAG = EpcMaskActivity.class.getSimpleName();

    private static final int DEFAULT_OFFSET = 16;
    private static final String DEFAULT_MASK = "";
    private static final int DEFAULT_LENGTH = 16;
    private static final int NIBBLE_SIZE = 4;
    private static final String UNIT_BIT = " bit";

    private static final int RESULT_DISCONNECTED = Activity.RESULT_FIRST_USER + 1;

    private AsReaderP3xU mAsReaderP3xU = null;
    private Button btnSave;
    private Button btnClear;

    private TextView tvOffset;
    private TextView tvMask;
    private TextView tvLength;
    private int mOffset;
    private String mMask;
    private int mLength;

    private MaskWheelDialog offsetDialog;
    private MaskWheelDialog lengthDialog;
    private List<String> lengthList;

    private Handler mEvent;

    private UserConfigs mUserConfigs;
    private AsReaderP3xUSelectMaskEpcParam selectMaskEpcParam;
    // ************************************************************************
    // Activity Event Methods
    // ************************************************************************

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_selection_mask_epc);
        mUserConfigs = UserConfigs.getInstance(this.getApplicationContext());
        ActionBar actionBar = getActionBar();
        if (actionBar != null) {
            actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
            actionBar.setCustomView(R.layout.activity_title);
            actionBar.setDisplayHomeAsUpEnabled(true);
            actionBar.setDisplayShowHomeEnabled(true);
        }
        TextView titleText = (TextView) findViewById(R.id.title_text);
        titleText.setText(R.string.select_epc_mask_item_name);
        final int version = Build.VERSION.SDK_INT;
        if (version >= Build.VERSION_CODES.M)
            titleText.setTextColor(getColor(getApplicationContext(), R.color.black));
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onCreate()");

        //_userMasks = new UserMasks(this.getApplicationContext());

        // Get RFID Reader Instance
        mAsReaderP3xU = AsReaderP3xUManager.getInstance();
        mAsReaderP3xU.setEventListener(this);

        // Initialize Widget
        initWidgets();
        // Enable action widget
        enableActionWidgets(false);

        mEvent = new Handler();

        // Load EPC Mask
        WaitDialog.show(this, R.string.load_EPC_mask);
        new Thread(new Runnable() {

            @Override
            public void run() {

                int count = 0;
                try {
                    count = mAsReaderP3xU.getEpcMaskCount();

                    if (count != 0) {
                        selectMaskEpcParam = mAsReaderP3xU.getEpcMask(0);
                        if (selectMaskEpcParam == null) {
                            selectMaskEpcParam = new AsReaderP3xUSelectMaskEpcParam();
                            selectMaskEpcParam.setOffset(16);
                            selectMaskEpcParam.setLength(16);
                        }
                    } else {
                        selectMaskEpcParam = new AsReaderP3xUSelectMaskEpcParam();
                        selectMaskEpcParam.setOffset(16);
                        selectMaskEpcParam.setLength(16);
                    }

                    mEvent.post(new Runnable() {

                        @Override
                        public void run() {
                            enableActionWidgets(true);
                            mOffset = selectMaskEpcParam.getOffset();
                            mMask = (selectMaskEpcParam.getMask() == null) ? "" : selectMaskEpcParam.getMask();
                            if (selectMaskEpcParam.getLength() == 0){
                                selectMaskEpcParam.setLength(16);
                            }
                            mLength = selectMaskEpcParam.getLength();

                            displayOffSet();
                            displayMask();
                            displayLength();
                            WaitDialog.hide();
                        }

                    });

                } catch (Exception e) {
                    AsReaderP3xUASRLog.e(TAG, e, "ERROR. initReader() - Failed to get init Epc Mask");
                    WaitDialog.hide();
                }
            }

        }).start();
    }

    @Override
    protected void onDestroy() {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onDestroy()");

        mAsReaderP3xU.removeEventListener(this);

        if (mAsReaderP3xU.getAction() != AsReaderP3xUActionState.Stop &&
                mAsReaderP3xU.getState() == AsReaderP3xUConnectionState.Connected) {
            mAsReaderP3xU.stop();
        }

        super.onDestroy();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onOptionsItemSelected(home)");
                setResult(Activity.RESULT_CANCELED);
                finish();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @SuppressWarnings("deprecation")
    @Override
    public void onClick(View v) {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));

        switch (v.getId()) {
            case R.id.tv_offset:
                offsetDialog.show();
                break;
            case R.id.tv_mask:
                showMaskDialog();
                break;
            case R.id.tv_length:
                lengthDialog.show();
                break;
            case R.id.save:

                if (TextUtils.isEmpty(mMask) || TextUtils.isEmpty(tvMask.getText().toString().trim())) {
                    return;
                }
                if (mLength == 0) {
                    Toast.makeText(this, R.string.length_error, Toast.LENGTH_SHORT).show();
                    return;
                }

                enableActionWidgets(false);
                WaitDialog.show(this, R.string.save_EPC_mask);
                new Thread(new Runnable() {

                    @Override
                    public void run() {
                        try {
                            AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "+++ INFO. onClick(save)$Runnable.run()");

                            AsReaderP3xUSelectMaskEpcParam item = new AsReaderP3xUSelectMaskEpcParam();
                            item.setOffset(mOffset);
                            item.setMask(mMask);
                            item.setLength(mLength);

                            //save epc
                            mAsReaderP3xU.addEpcMask(item);

                            mUserConfigs.clearEpcMasks();
                            mUserConfigs.addEpcMask(item);
                            mUserConfigs.Save();

                            mEvent.post(new Runnable() {

                                @Override
                                public void run() {
                                    AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onClick(save)$Runnable.run()$Runable.run()");
                                    WaitDialog.hide();
                                    finish();
                                }

                            });

                        } catch (AsReaderP3xUException e) {
                            e.printStackTrace();
                            AsReaderP3xUASRLog.e(TAG, e, "--- ERROR. onClick(save)$Runnable.run() - Failed to save select mask EPC" + e.getMessage());
                            WaitDialog.hide();
                            finish();
                            return;
                        }
                        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "--- INFO. onClick(save)$Runnable.run()");
                    }

                }).start();
                break;
            case R.id.btn_clear:

                mUserConfigs.clearEpcMasks();
                mUserConfigs.Save();
                try {
                    mAsReaderP3xU.clearEpcMask();
                } catch (AsReaderP3xUException e) {
                    e.printStackTrace();
                }

                mOffset = DEFAULT_OFFSET;
                mMask = DEFAULT_MASK;
                mLength = DEFAULT_LENGTH;

                displayOffSet();
                displayMask();
                displayLength();

                break;
        }
    }


    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onItemClick(mask_list, %d, %d)", position, id);
        enableActionWidgets(true);
    }

    @Override
    public void onClick(DialogInterface dialog, int which) {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. DialogInterface.onClick()");
    }

    // ************************************************************************
    // RFID Reader Event Methods
    // ************************************************************************
    @Override
    public void onStateChanged(AsReaderP3xU asReaderP3xU, AsReaderP3xUConnectionState state) {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onStateChanged(%s)", state);
        if (state == AsReaderP3xUConnectionState.Disconnected) {
            setResult(RESULT_DISCONNECTED);
            finish();
        }
    }

    @Override
    public void onActionChanged(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action) {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onActionChanged(%s)", action);
    }

    @Override
    public void onReadTag(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action, String tag, float rssi) {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onReadedTag(%s, [%s], %.2f)", action, tag, rssi);
    }

    @Override
    public void onAccessResult(AsReaderP3xU asReaderP3xU, AsReaderP3xUResultCode code, AsReaderP3xUActionState action, String epc, String data) {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onAccessResult(%s, %s, [%s], [%s], %.2f)", code, action, epc, data);
    }

//	@Override
//	public void onDebugMessage(AsReaderP3xU asReaderP3xU, String msg) {
//		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onDebugMessage([%s])", msg);
//	}


//	@Override
//	public void onRemoteKeyStateChanged(AsReaderP3xU asReaderP3xU, RemoteKeyState state) {
//		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onRemoteKeyStateChanged(%s)", state);
//	}

    // ************************************************************************
    // Helper Methods
    // ************************************************************************

    // Initialize Widgets
    private void initWidgets() {
        btnSave = (Button) findViewById(R.id.save);
        btnSave.setOnClickListener(this);

        btnClear = (Button) findViewById(R.id.btn_clear);
        btnClear.setOnClickListener(this);

        tvOffset = (TextView) findViewById(R.id.tv_offset);
        tvOffset.setOnClickListener(this);


//		etMask.addTextChangedListener(new TextWatcher() {
//			@Override
//			public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) { }
//
//			@Override
//			public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) { }
//
//			@Override
//			public void afterTextChanged(Editable editable) {
//
//				if (editable == null) {
//					return;
//				}
//
//				String s = editable.toString();
//				if (s.length() > mLength / 4) {
//					Toast.makeText(EpcMaskActivity.this, getString(R.string.length_error), Toast.LENGTH_SHORT).show();
//
//				} else {
//					// mMask = editable.toString();
//					// mLength = mMask.length() * NIBBLE_SIZE;
//					displayLength();
//
//				}
//
//
//			}
//		});


        // etMask.setOnClickListener(this);


        // tvLength.setText(String.format(Locale.US, "%d", tvMask.getText().toString().trim().length() * NIBBLE_SIZE));
        // mLength = mMask.length() * NIBBLE_SIZE;

        tvMask = (TextView) findViewById(R.id.tv_mask);
        tvMask.setOnClickListener(this);
        tvMask.setText("");

        tvLength = (TextView) findViewById(R.id.tv_length);
        tvLength.setOnClickListener(this);

        initLengthDialog();

        mOffset = DEFAULT_OFFSET;
        mMask = DEFAULT_MASK;
        mLength = DEFAULT_LENGTH;

        displayOffSet();
        displayMask();
        displayLength();
    }

    private void initLengthDialog() {

        lengthList = new ArrayList<>();
        for (int i = 0; i < DEFAULT_OFFSET; i++) {
            lengthList.add(i, String.valueOf(i * DEFAULT_OFFSET) + UNIT_BIT);
        }

        offsetDialog = new MaskWheelDialog(this);
        offsetDialog.setTitle(getString(R.string.length))
                .setButtonText("OK")
                .setDialogStyle(getColor(R.color.black)) // 选中文字的颜色
                .setCount(3)
                .setOnDialogItemClickListener(new WheelViewDialog.OnDialogItemClickListener() {
                    @Override
                    public void onItemClick(int position, String s) {

//						if ("0".equals(s.trim())) {
//							return;
//						}

                        String str = s.substring(0, s.length() - 4).trim();
                        mOffset = Integer.parseInt(str);
                        displayOffSet();

                    }
                });

        offsetDialog.setListItems(lengthList);

        lengthDialog = new MaskWheelDialog(this);
        lengthDialog.setTitle(getString(R.string.length))
                .setButtonText("OK")
                .setDialogStyle(getColor(R.color.black)) // 选中文字的颜色
                .setCount(3)
                .setOnDialogItemClickListener(new WheelViewDialog.OnDialogItemClickListener() {
                    @Override
                    public void onItemClick(int position, String s) {

                        if ("0".equals(s.trim())) {
                            tvMask.setText("");
                            mMask = "";

                            return;
                        }

                        String str = s.substring(0, s.length() - 4).trim();
                        mLength = Integer.parseInt(str);
                        // tvLength.setText(s);
                        displayLength();

                    }
                });

        lengthDialog.setListItems(lengthList);
    }

    // Enable/Disable Action Widgets
    private void enableActionWidgets(boolean enabled) {
        btnSave.setEnabled(enabled);
        btnClear.setEnabled(enabled);
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
    }

    @Override
    public void onKeyEvent(AsReaderP3xU asReaderP3xU, AsReaderP3xUKeyType key, AsReaderP3xUKeyState state) {

    }

    private void showMaskDialog() {

        LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_number_16, null);

        final EditText etInput = (EditText) root.findViewById(R.id.et_mask_input);

//		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(0, 255) };
//		etInput.setFilters(filters);

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.mask);
        builder.setView(root);
        builder.setPositiveButton(R.string.action_ok, null);

        // 点对话框以外的时候触发
        builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if (imm != null)
                    imm.hideSoftInputFromWindow(etInput.getWindowToken(), 0);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showAutoOffTimeDialog().$NegativeButton.onClick()");
            }
        });
        builder.setCancelable(true);

        // 点取消时触发
        builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

            @Override
            public void onCancel(DialogInterface dialog) {
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if (imm != null)
                    imm.hideSoftInputFromWindow(etInput.getWindowToken(), 0);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showAutoOffTimeDialog().onCancel()");
            }
        });

        AlertDialog dialog = builder.create();

        dialog.setOnShowListener(new DialogInterface.OnShowListener() {

            @Override
            public void onShow(DialogInterface dialog) {
                // 显示(对话框刚出来)的时候执行
                final AlertDialog dlg = (AlertDialog) dialog;
                etInput.setText(String.format(Locale.US, "%s", mMask));
                etInput.selectAll();

                //	etInput.setKeyListener(DigitsKeyListener.getInstance(digists));
                // Edittext.setInputType(InputType.TYPE_CLASS_TEXT);
//				etInput.setInputType(InputType.TYPE_TEXT_VARIATION_EMAIL_ADDRESS); // 英文键盘
//				etInput.setImeOptions(EditorInfo.IME_ACTION_DONE); // 输入法右下角的按钮变"完成"

                etInput.setKeyListener(new DigitsKeyListener() {
                    @Override
                    public int getInputType() {
                        return InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_NORMAL; // 弹英文键盘
                    }

                    @Override
                    protected char[] getAcceptedChars() {
                        return "0123456789abcdefABCDEF".toCharArray(); // 限制 16 进制
                    }
                });

                etInput.requestFocus();

                // OK 按钮
                Button positiveButton = dlg.getButton(AlertDialog.BUTTON_POSITIVE);
                positiveButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        if (TextUtils.isEmpty(etInput.getText().toString())) {
                            InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                            if (imm != null) {
                                imm.hideSoftInputFromWindow(etInput.getWindowToken(), 0);
                            }
                            Toast.makeText(EpcMaskActivity.this, getString(R.string.length_error), Toast.LENGTH_SHORT).show();
                            dlg.dismiss();
                            return;
                        }

                        try {
                            String inputStr = etInput.getText().toString();
                            if (inputStr.length() != mLength / NIBBLE_SIZE) {
                                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                                if (imm != null) {
                                    imm.hideSoftInputFromWindow(etInput.getWindowToken(), 0);
                                }
                                // dlg.dismiss();
                                Toast.makeText(EpcMaskActivity.this, getString(R.string.length_error), Toast.LENGTH_SHORT).show();
                            } else {
                                mMask = inputStr;
                            }

                            // mOffset = Integer.parseInt(etInput.getText().toString().trim());
                        } catch (Exception e) {
                            mLength = 0;
                            tvLength.setText("0");
                            mMask = "";
                            // tvMask.setText("");
                        }

                        displayMask();

                        InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                        if (imm != null)
                            imm.hideSoftInputFromWindow(etInput.getWindowToken(), 0);
                        dlg.dismiss();
                    }
                });
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if (imm != null)
                    imm.showSoftInput(etInput, InputMethodManager.SHOW_FORCED);
                AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showAutoOffTimeDialog().onShow()");
            }
        });
        dialog.show();
    }

    private void displayOffSet() {
        tvOffset.setText(String.format(Locale.US, "%d%s", mOffset, UNIT_BIT));
    }

    private void displayMask() {
        tvMask.setText(String.format(Locale.US, "%s", mMask));
    }

    private void displayLength() {
        tvLength.setText(String.format(Locale.US, "%d%s", mLength, UNIT_BIT));
    }

    @SuppressLint("NewApi")
    @SuppressWarnings("deprecation")
    protected int getColor(Context context, int id) {
        final int version = Build.VERSION.SDK_INT;
        if (version >= Build.VERSION_CODES.M)
            return context.getColor(id);
        else
            return context.getResources().getColor(id);
    }
}
